@@ -1,4 +1,4 @@
-/* $Header: /p/tcsh/cvsroot/tcsh/sh.set.c,v 3.83 2012/01/15 17:15:28 christos Exp $ */
+/* $Header: /p/tcsh/cvsroot/tcsh/sh.set.c,v 3.86 2014/10/28 18:40:46 christos Exp $ */
 /*
  * sh.set.c: Setting and Clearing of variables
  */
@@ -32,7 +32,7 @@
  */
 #include "sh.h"
 
-RCSID("$tcsh: sh.set.c,v 3.83 2012/01/15 17:15:28 christos Exp $")
+RCSID("$tcsh: sh.set.c,v 3.86 2014/10/28 18:40:46 christos Exp $")
 
 #include "ed.h"
 #include "tw.h"
@@ -195,6 +195,9 @@ update_vars(Char *vp)
     else if (eq(vp, STRkillring)) {
 	SetKillRing((int)getn(varval(vp)));
     }
+    else if (eq(vp, STRhistory)) {
+	sethistory((int)getn(varval(vp)));
+    }
 #ifndef HAVENOUTMP
     else if (eq(vp, STRwatch)) {
 	resetwatch();
@@ -203,6 +206,9 @@ update_vars(Char *vp)
     else if (eq(vp, STRimplicitcd)) {
 	implicit_cd = ((eq(varval(vp), STRverbose)) ? 2 : 1);
     }
+    else if (eq(vp, STRcdtohome)) {
+	cdtohome = 1;
+    }
 #ifdef COLOR_LS_F
     else if (eq(vp, STRcolor)) {
 	set_color_context();
@@ -780,12 +786,16 @@ unset(Char **v, struct command *c)
 	symlinks = 0;
     if (adrof(STRimplicitcd) == 0)
 	implicit_cd = 0;
+    if (adrof(STRcdtohome) == 0)
+	cdtohome = 0;
     if (adrof(STRkillring) == 0)
 	SetKillRing(0);
     if (did_edit && noediting && adrof(STRedit) == 0)
 	noediting = 0;
     if (did_roe && adrof(STRrecognize_only_executables) == 0)
 	tw_cmd_free();
+    if (adrof(STRhistory) == 0)
+	sethistory(0);
 #ifdef COLOR_LS_F
     if (adrof(STRcolor) == 0)
 	set_color_context();
